(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory((global.index = global.index || {}, global.index.ts = {})));
}(this, (function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }

    var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

    function unwrapExports (x) {
    	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
    }

    function createCommonjsModule(fn, module) {
    	return module = { exports: {} }, fn(module, module.exports), module.exports;
    }

    var lib = createCommonjsModule(function (module, exports) {
    var __read = (commonjsGlobal && commonjsGlobal.__read) || function (o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    };
    var __spread = (commonjsGlobal && commonjsGlobal.__spread) || function () {
        for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
        return ar;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * `pipe` composes a series of functions passing the output of on function into the input of the next.
     * The first function provided to pipe can of any arity, however, each subsequent function can only accept
     * one parameter. Functions are read from right to left.
     *
     * In the event that the outgoing type doesn't match in the next functions incoming type, a compiler
     * error is raised and the type of the assigned object will have information on where the error occurred.
     * In an TypeScript aware editor, mouse over the object assigned to the `pipe` composition outcome to see result.
     * @example
     * const longerWord = ( word1: string, word2: string ) => word1.length > word2.length ? word1 : word2
     * const longestWord = ( name: string, ...names: string[]) => [name,...names].reduce(longerWord, '')
     * const length = ( xs: string | unknown[] ) => xs.length
     * const longestNameLength = pipe(
     *  longestWord,
     *  length,
     * )
     */
    exports.pipe = function (entry) {
        var funcs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            funcs[_i - 1] = arguments[_i];
        }
        return (function () {
            var arg = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                arg[_i] = arguments[_i];
            }
            return funcs.reduce(function (acc, item) { return item.call(item, acc); }, entry.apply(void 0, __spread(arg)));
        });
    };
    /**
     * Pipeline is simply pipe immediately invoked. The benefit is moving the
     * parameter passed to `pipe` to the first parameter rather than the last.
     * The improves readability. Data flows from the top to the bottom.
     *
     * Like `pipe` in the event of a compiler error arising from a type mismatch in the
     * provided function's input or output, the returned type will be an object
     * which explains where the error occurs.
     * @example
     * const add = (a: number) => (b: number) => a + b
     * const multiply = ( a: number ) => (b: number ) => a * b
     * const double = multiply(2)
     * const half = multiply(.5)
     * const ten = pipeline(
     *   2,
     *   add(3),
     *   double,
     *   double,
     *   half,
     * )
     */
    exports.pipeline = function (arg, first) {
        var rest = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            rest[_i - 2] = arguments[_i];
        }
        return exports.pipe.apply(void 0, __spread([first], rest))(arg);
    };
    var init = function (xs) { return xs.slice(0, -1); };
    var last = function (xs) { return xs.slice(-1)[0]; };
    /**
     * `compose` composes a series of functions passing the output of on function into the input of the next.
     * The last function provided to pipe can of any arity, however, each subsequent function can only accept
     * one parameter. Functions are read from left to right.
     *
     * In the event that the outgoing type doesn't match in the next functions incoming type, a compiler
     * error is raised and the type of the assigned object will have information on where the error occurred.
     * In an TypeScript aware editor, mouse over the object assigned to the `compose` composition outcome to see result.
     * @example
     * const longerWord = ( word1: string, word2: string ) => word1.length > word2.length ? word1 : word2
     * const longestWord = ( name: string, ...names: string[]) => [name,...names].reduce(longerWord, '')
     * const length = ( xs: string | unknown[] ) => xs.length
     * const longestNameLength = pipe(
     *  length,
     *  longestWord,
     * )
     */
    exports.compose = function (first) {
        var funcs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            funcs[_i - 1] = arguments[_i];
        }
        /* `any` is used as return type because on compile error we present an object,
            which will not match this */
        return function () {
            var arg = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                arg[_i] = arguments[_i];
            }
            return init(__spread([first], funcs)).reduceRight(function (acc, item) { return item.call(item, acc); }, last(funcs).apply(void 0, __spread(arg)));
        };
    };
    });

    unwrapExports(lib);
    var lib_1 = lib.pipe;
    var lib_2 = lib.pipeline;
    var lib_3 = lib.compose;

    var objectPath = createCommonjsModule(function (module) {
    (function (root, factory){

      /*istanbul ignore next:cant test*/
      {
        module.exports = factory();
      }
    })(commonjsGlobal, function(){

      var toStr = Object.prototype.toString;
      function hasOwnProperty(obj, prop) {
        if(obj == null) {
          return false
        }
        //to handle objects with null prototypes (too edge case?)
        return Object.prototype.hasOwnProperty.call(obj, prop)
      }

      function isEmpty(value){
        if (!value) {
          return true;
        }
        if (isArray(value) && value.length === 0) {
            return true;
        } else if (typeof value !== 'string') {
            for (var i in value) {
                if (hasOwnProperty(value, i)) {
                    return false;
                }
            }
            return true;
        }
        return false;
      }

      function toString(type){
        return toStr.call(type);
      }

      function isObject(obj){
        return typeof obj === 'object' && toString(obj) === "[object Object]";
      }

      var isArray = Array.isArray || function(obj){
        /*istanbul ignore next:cant test*/
        return toStr.call(obj) === '[object Array]';
      };

      function isBoolean(obj){
        return typeof obj === 'boolean' || toString(obj) === '[object Boolean]';
      }

      function getKey(key){
        var intKey = parseInt(key);
        if (intKey.toString() === key) {
          return intKey;
        }
        return key;
      }

      function factory(options) {
        options = options || {};

        var objectPath = function(obj) {
          return Object.keys(objectPath).reduce(function(proxy, prop) {
            if(prop === 'create') {
              return proxy;
            }

            /*istanbul ignore else*/
            if (typeof objectPath[prop] === 'function') {
              proxy[prop] = objectPath[prop].bind(objectPath, obj);
            }

            return proxy;
          }, {});
        };

        function hasShallowProperty(obj, prop) {
          return (options.includeInheritedProps || (typeof prop === 'number' && Array.isArray(obj)) || hasOwnProperty(obj, prop))
        }

        function getShallowProperty(obj, prop) {
          if (hasShallowProperty(obj, prop)) {
            return obj[prop];
          }
        }

        function set(obj, path, value, doNotReplace){
          if (typeof path === 'number') {
            path = [path];
          }
          if (!path || path.length === 0) {
            return obj;
          }
          if (typeof path === 'string') {
            return set(obj, path.split('.').map(getKey), value, doNotReplace);
          }
          var currentPath = path[0];
          var currentValue = getShallowProperty(obj, currentPath);
          if (path.length === 1) {
            if (currentValue === void 0 || !doNotReplace) {
              obj[currentPath] = value;
            }
            return currentValue;
          }

          if (currentValue === void 0) {
            //check if we assume an array
            if(typeof path[1] === 'number') {
              obj[currentPath] = [];
            } else {
              obj[currentPath] = {};
            }
          }

          return set(obj[currentPath], path.slice(1), value, doNotReplace);
        }

        objectPath.has = function (obj, path) {
          if (typeof path === 'number') {
            path = [path];
          } else if (typeof path === 'string') {
            path = path.split('.');
          }

          if (!path || path.length === 0) {
            return !!obj;
          }

          for (var i = 0; i < path.length; i++) {
            var j = getKey(path[i]);

            if((typeof j === 'number' && isArray(obj) && j < obj.length) ||
              (options.includeInheritedProps ? (j in Object(obj)) : hasOwnProperty(obj, j))) {
              obj = obj[j];
            } else {
              return false;
            }
          }

          return true;
        };

        objectPath.ensureExists = function (obj, path, value){
          return set(obj, path, value, true);
        };

        objectPath.set = function (obj, path, value, doNotReplace){
          return set(obj, path, value, doNotReplace);
        };

        objectPath.insert = function (obj, path, value, at){
          var arr = objectPath.get(obj, path);
          at = ~~at;
          if (!isArray(arr)) {
            arr = [];
            objectPath.set(obj, path, arr);
          }
          arr.splice(at, 0, value);
        };

        objectPath.empty = function(obj, path) {
          if (isEmpty(path)) {
            return void 0;
          }
          if (obj == null) {
            return void 0;
          }

          var value, i;
          if (!(value = objectPath.get(obj, path))) {
            return void 0;
          }

          if (typeof value === 'string') {
            return objectPath.set(obj, path, '');
          } else if (isBoolean(value)) {
            return objectPath.set(obj, path, false);
          } else if (typeof value === 'number') {
            return objectPath.set(obj, path, 0);
          } else if (isArray(value)) {
            value.length = 0;
          } else if (isObject(value)) {
            for (i in value) {
              if (hasShallowProperty(value, i)) {
                delete value[i];
              }
            }
          } else {
            return objectPath.set(obj, path, null);
          }
        };

        objectPath.push = function (obj, path /*, values */){
          var arr = objectPath.get(obj, path);
          if (!isArray(arr)) {
            arr = [];
            objectPath.set(obj, path, arr);
          }

          arr.push.apply(arr, Array.prototype.slice.call(arguments, 2));
        };

        objectPath.coalesce = function (obj, paths, defaultValue) {
          var value;

          for (var i = 0, len = paths.length; i < len; i++) {
            if ((value = objectPath.get(obj, paths[i])) !== void 0) {
              return value;
            }
          }

          return defaultValue;
        };

        objectPath.get = function (obj, path, defaultValue){
          if (typeof path === 'number') {
            path = [path];
          }
          if (!path || path.length === 0) {
            return obj;
          }
          if (obj == null) {
            return defaultValue;
          }
          if (typeof path === 'string') {
            return objectPath.get(obj, path.split('.'), defaultValue);
          }

          var currentPath = getKey(path[0]);
          var nextObj = getShallowProperty(obj, currentPath);
          if (nextObj === void 0) {
            return defaultValue;
          }

          if (path.length === 1) {
            return nextObj;
          }

          return objectPath.get(obj[currentPath], path.slice(1), defaultValue);
        };

        objectPath.del = function del(obj, path) {
          if (typeof path === 'number') {
            path = [path];
          }

          if (obj == null) {
            return obj;
          }

          if (isEmpty(path)) {
            return obj;
          }
          if(typeof path === 'string') {
            return objectPath.del(obj, path.split('.'));
          }

          var currentPath = getKey(path[0]);
          if (!hasShallowProperty(obj, currentPath)) {
            return obj;
          }

          if(path.length === 1) {
            if (isArray(obj)) {
              obj.splice(currentPath, 1);
            } else {
              delete obj[currentPath];
            }
          } else {
            return objectPath.del(obj[currentPath], path.slice(1));
          }

          return obj;
        };

        return objectPath;
      }

      var mod = factory();
      mod.create = factory;
      mod.withInheritedProps = factory({includeInheritedProps: true});
      return mod;
    });
    });

    var path = /*#__PURE__*/Object.freeze({
        __proto__: null,
        'default': objectPath,
        __moduleExports: objectPath
    });

    // Transform an input based on a map of transform actions
    var transform = function (input, map) {
        // Use Reduce
        return map.reduce(function (o, _a) {
            var sourcePath = _a[0], targetPath = _a[1], transforms = _a.slice(2);
            // Create a new transform stack that will process the current value
            var transform = lib_1.apply(void 0, __spreadArrays([function () {
                    return Array.isArray(sourcePath)
                        ? sourcePath.map(function (k) { return undefined(input, k); })
                        : undefined(input, sourcePath);
                }], transforms));
            // Execute the pipline and set the result to the target key on the output object
            undefined(o, targetPath, transform());
            // Return the
            return o;
        }, {});
    };

    exports.transform = transform;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=data-transformer.umd.js.map
